@@ -1,3 +1,16 @@
+// Copyright 2015 The Prometheus Authors
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+// http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
 // +build !nonetstat
 
 package collector
@@ -14,65 +27,53 @@ import (
 )
 
 const (
-	procNetStat       = "/proc/net/netstat"
-	procSNMPStat      = "/proc/net/snmp"
 	netStatsSubsystem = "netstat"
 )
 
-type netStatCollector struct {
-	metrics map[string]prometheus.Gauge
-}
+type netStatCollector struct{}
 
 func init() {
 	Factories["netstat"] = NewNetStatCollector
 }
 
-// NewNetStatCollector takes a returns
+// NewNetStatCollector takes and returns
 // a new Collector exposing network stats.
 func NewNetStatCollector() (Collector, error) {
-	return &netStatCollector{
-		metrics: map[string]prometheus.Gauge{},
-	}, nil
+	return &netStatCollector{}, nil
 }
 
 func (c *netStatCollector) Update(ch chan<- prometheus.Metric) (err error) {
-	netStats, err := getNetStats(procNetStat)
+	netStats, err := getNetStats(procFilePath("net/netstat"))
 	if err != nil {
 		return fmt.Errorf("couldn't get netstats: %s", err)
 	}
-	snmpStats, err := getNetStats(procSNMPStat)
+	snmpStats, err := getNetStats(procFilePath("net/snmp"))
 	if err != nil {
 		return fmt.Errorf("couldn't get SNMP stats: %s", err)
 	}
 	// Merge the results of snmpStats into netStats (collisions are possible, but
-	// we know that the keys are always unique for the given use case)
+	// we know that the keys are always unique for the given use case).
 	for k, v := range snmpStats {
 		netStats[k] = v
 	}
 	for protocol, protocolStats := range netStats {
 		for name, value := range protocolStats {
 			key := protocol + "_" + name
-			if _, ok := c.metrics[key]; !ok {
-				c.metrics[key] = prometheus.NewGauge(
-					prometheus.GaugeOpts{
-						Namespace: Namespace,
-						Subsystem: netStatsSubsystem,
-						Name:      key,
-						Help:      fmt.Sprintf("%s %s from /proc/net/{netstat,snmp}.", protocol, name),
-					},
-				)
-			}
 			v, err := strconv.ParseFloat(value, 64)
 			if err != nil {
 				return fmt.Errorf("invalid value %s in netstats: %s", value, err)
 			}
-			c.metrics[key].Set(v)
+			ch <- prometheus.MustNewConstMetric(
+				prometheus.NewDesc(
+					prometheus.BuildFQName(Namespace, netStatsSubsystem, key),
+					fmt.Sprintf("Protocol %s statistic %s.", protocol, name),
+					nil, nil,
+				),
+				prometheus.UntypedValue, v,
+			)
 		}
 	}
-	for _, m := range c.metrics {
-		m.Collect(ch)
-	}
-	return err
+	return nil
 }
 
 func getNetStats(fileName string) (map[string]map[string]string, error) {
